import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/coupons_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/coupon_create_user_coupons_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/coupon_get_coupons_query.dart';
import 'package:dio/dio.dart';

abstract class CouponRemoteDataSource {
  Future<Either<Failure, dynamic>> getCoupons({
    int? offset,
    int? limit,
    String? sortBy,
    List<String>? companyTokens,
    List<String>? retailerTokens,
  });

  Future<Either<Failure, dynamic>> createUserCoupons({
    required List<String> promotionTokens,
    String? latitude,
    String? longitude,
    String? saveMethod,
    String? utmSource,
    String? transId,
    String? crmId,
    required String publisherToken,
  });

  Future<Either<Failure, dynamic>> getCouponDetail({
    required String couponId,
  });

  Future<Either<Failure, dynamic>> activateCoupon({
    required String couponId,
  });

  Future<Either<Failure, dynamic>> deactivateCoupon({
    required String couponId,
  });

  Future<Either<Failure, dynamic>> getUserCouponsRetailers();

  Future<Either<Failure, dynamic>> getUserCouponsCompanies();
}

class CouponRemoteDataSourceImpl implements CouponRemoteDataSource {
  Dio dio = Dio()..interceptors.addAll(ConfigReader.dioInterceptors);
  // ..interceptors.add(LoggingInterceptor());

  @override
  Future<Either<Failure, dynamic>> getCoupons({
    int? offset,
    int? limit,
    String? sortBy,
    List<String>? companyTokens,
    List<String>? retailerTokens,
  }) async {
    dio.options.headers = CouponsApiHelper.buildHeaders();

    final queryParameters = CouponGetCouponsQuery(
      offset: offset,
      limit: limit,
      sortBy: sortBy,
      retailerTokens: retailerTokens,
      companyTokens: companyTokens,
    );

    try {
      final response = await dio.get(
        CouponsApiHelper.getCouponsListUrl(),
        queryParameters: queryParameters.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> createUserCoupons({
    required List<String> promotionTokens,
    String? latitude,
    String? longitude,
    String? saveMethod,
    String? utmSource,
    String? transId,
    String? crmId,
    required String publisherToken,
  }) async {
    dio.options.headers = CouponsApiHelper.buildHeaders();

    final couponCreateUserCouponsRequest = CouponCreateUserCouponsRequest(
      promotionTokens: promotionTokens,
      latitude: latitude,
      longitude: longitude,
      saveMethod: saveMethod,
      utmSource: utmSource,
      transId: transId,
      crmId: crmId,
      publisherToken: publisherToken,
    );

    try {
      final response = await dio.post(
        CouponsApiHelper.getCreateUserCouponsUrl(),
        data: couponCreateUserCouponsRequest.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> activateCoupon({
    required String couponId,
  }) async {
    dio.options.headers = CouponsApiHelper.buildHeaders();

    try {
      final response = await dio.post(
        CouponsApiHelper.getActivateCouponUrl(couponId),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> deactivateCoupon(
      {required String couponId}) async {
    dio.options.headers = CouponsApiHelper.buildHeaders();

    try {
      final response = await dio.post(
        CouponsApiHelper.getDeactivateCouponUrl(couponId),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getCouponDetail({
    required String couponId,
  }) async {
    dio.options.headers = CouponsApiHelper.buildHeaders();

    try {
      final response =
          await dio.get(CouponsApiHelper.getCouponDetailUrl(couponId));
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserCouponsCompanies() async {
    dio.options.headers = CouponsApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        CouponsApiHelper.getUserCouponsCompanies(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserCouponsRetailers() async {
    dio.options.headers = CouponsApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        CouponsApiHelper.getUserCouponsRetailers(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}
