import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_response.dart';

import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/third_party_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/users_api_helper.dart';

import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_update_user_data_request.dart';
import 'package:dio/dio.dart';

abstract class UserRemoteDataSource {
  Future<Either<Failure, String>> logIn({
    required String externalId,
    String? referralCode,
  });

  Future<Either<Failure, dynamic>> getUserData();

  Future<Either<Failure, dynamic>> updateUserData({
    required String alias,
    required String gender,
    required String birthdate,
  });

  Future<Either<Failure, dynamic>> unregisterUser();
}

class UserRemoteDataSourceImpl implements UserRemoteDataSource {
  Dio dio = Dio()..interceptors.addAll(ConfigReader.dioInterceptors);

  @override
  Future<Either<Failure, String>> logIn({
    required String externalId,
    String? referralCode,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = ThirdPartyAPIHelper.buildHeaders();

    final userLoginRequest = UserLoginRequest(
      externalId: externalId,
      referralCode: referralCode,
    );

    try {
      final response = await dio.post(
        ThirdPartyAPIHelper.loginUrl(),
        data: userLoginRequest.toJson(),
      );

      UserLoginResponse userLoginResponse =
          UserLoginResponse.fromResponse(response);
      return Right(userLoginResponse.sessionToken);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserData() async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        UsersApiHelper.getUserDataUrl(),
      );

      final data = response.data as Map<String, dynamic>?;

      return Right(data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> updateUserData({
    required String alias,
    required String gender,
    required String birthdate,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = UsersApiHelper.buildHeaders();

    final userUpdateUserDataRequest = UserUpdateUserDataRequest(
      alias: alias,
      gender: gender,
      birthdate: birthdate,
    );

    try {
      final response = await dio.put(
        UsersApiHelper.updateUserDataUrl(),
        data: userUpdateUserDataRequest.toJson(),
      );

      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> unregisterUser() async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    try {
      await dio.post(
        UsersApiHelper.unregisterUserUrl(),
      );

      return Right(true);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}
